package com.voltage.carl.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.ManyToMany;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.swing.tree.TreeNode;

@Entity
@NamedQueries({
	@NamedQuery(name="Product.findAll", query="select p from Product p order by p.family, p.name"),
	@NamedQuery(name="Product.count", query="select count(p) from Product p"),
	@NamedQuery(name="Product.findByID", query="select p from Product p where p.id = :id"),
	@NamedQuery(name="Product.findByName", query="select p from Product p where p.name = :name"),
	@NamedQuery(name="Product.findByFamily", query="select p from Product p where p.family = :family")
})
public class Product extends BaseEntity implements Comparable<Product>, TreeNode{
	
	@Column(nullable=false)
	@Enumerated(EnumType.STRING)
	private ProductFamily family;

	@Column(nullable=false, unique=true)
	private String name;

	@OneToMany(mappedBy="product")
	private List<Version> versions;
	
	@ManyToMany
	private List<Customer> customers;
	
	public Product() {
		//blank
	}
	
	public Product(ProductFamily family, String name) {
		setFamily(family);
		setName(name);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public ProductFamily getFamily() {
		return family;
	}
	
	public void setFamily(ProductFamily pf) {
		this.family = pf;
	}
	
	public List<Version> getVersions() {
		if (versions == null) {
			versions = new ArrayList<Version>();
		}
		Collections.sort(versions);
		return versions;
	}
	
	public void addVersion(Version version) {
		getVersions().add(version);
	}
	
	public Version getVersion(String versionNumber) {
		// inefficient, but we're dealing with just a few versions per product
		Version v = null;
		for (Version version : getVersions()) {
			if (version.getVersionNumber().equals(versionNumber)) {
				v = version;
				break;
			}
		}
		return v;
	}

	// convenience method to get all the SKUs for a product
	public List<SKU> getSKUs() {
		List<SKU> skus = new ArrayList<SKU>();
		for (Version version : getVersions()) {
			skus.addAll(version.getSKUs());
		}
		return skus;
	}
	
	public List<Customer> getCustomers() {
		if (customers == null) {
			customers = new ArrayList<Customer>();
		}
		return customers;
	}

	public void setCustomers(List<Customer> customers) {
		this.customers = customers;
	}
	
	public void addCustomer(Customer c) {
		getCustomers().add(c);
	}



	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!super.equals(obj)) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Product other = (Product) obj;
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "Product [id=" + id + ", name=" + name + "]";
	}

	public String getDisplayString() {
		return getName();
	}

	@Override
	public int compareTo(Product other) {
		//first sort by family
		
		if (family.equals(other.family)) {
			return name.toLowerCase().compareTo(other.name.toLowerCase());
		} else {
			return family.compareTo(other.family);
		}
		
	}

	///////////
	//
	// TreeNode methods
	//
	//////////
	
	@Override
	public Enumeration<Version> children() {
		return Collections.enumeration(getVersions());
	}

	@Override
	public boolean getAllowsChildren() {
		return true;
	}

	@Override
	public TreeNode getChildAt(int childIndex) {
		return getVersions().get(childIndex);
	}

	@Override
	public int getChildCount() {
		return getVersions().size();
	}

	@Override
	public int getIndex(TreeNode node) {
		return getVersions().indexOf(node);
	}

	@Override
	public TreeNode getParent() {
		return family;
	}

	@Override
	public boolean isLeaf() {
		return getVersions().isEmpty();
	}

	public String getType() {
		return "Product";
	}

	
}
